from django.apps import apps as django_apps
from django.views.generic.base import TemplateView
from edc_dashboard.utils import get_bootstrap_version
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from ..constants import CLINIC_LABEL_PRINTER, LAB_LABEL_PRINTER, PRINT_SERVER_NAME
from ..view_mixins import EdcLabelViewMixin


class PrinterSetupView(EdcViewMixin, NavbarViewMixin, EdcLabelViewMixin, TemplateView):
    template_name = f"edc_label/bootstrap{get_bootstrap_version()}/printer_setup.html"
    navbar_name = "edc_label"
    navbar_selected_item = "label"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            CLINIC_LABEL_PRINTER=CLINIC_LABEL_PRINTER,
            LAB_LABEL_PRINTER=LAB_LABEL_PRINTER,
            PRINT_SERVER_NAME=PRINT_SERVER_NAME,
            label_templates=list(
                django_apps.get_app_config("edc_label").label_templates.keys()
            ),
        )
        return context



import os
import sys

from django.apps import AppConfig as DjangoAppConfig
from django.conf import settings
from django.core.management.color import color_style

style = color_style()


class EdcLabelAppConfigError(Exception):
    pass


class AppConfig(DjangoAppConfig):
    name = "edc_label"

    verbose_name = "Edc Label"
    include_in_administration_section = True
    # default extension
    template_ext = "lbl"

    label_templates = {}

    def ready(self):
        sys.stdout.write(f"Loading {self.verbose_name} ...\n")
        try:
            label_template_folder = settings.LABEL_TEMPLATE_FOLDER
        except AttributeError:
            label_template_folder = os.path.join(settings.BASE_DIR, "label_templates")
            sys.stdout.write(" Using default label template path,)\n")
            sys.stdout.write(" to customize set LABEL_TEMPLATE_FOLDER in settings.\n")
        sys.stdout.write(" Label template folder is '{label_template_folder}'.\n")
        if not os.path.exists(label_template_folder):
            sys.stdout.write(
                style.ERROR(
                    "Label template folder does not exist!\n" "Not loading label templates\n"
                )
            )
        else:
            for filename in os.listdir(label_template_folder):
                if filename.endswith(self.template_ext):
                    label_name = filename.split(".")[0]
                    self.label_templates.update(
                        {label_name: os.path.join(label_template_folder, filename)}
                    )
                    sys.stdout.write(f" * {filename}\n")
        sys.stdout.write(f" Done loading {self.verbose_name}.\n")



from typing import Optional

from django.apps import apps as django_apps
from edc_protocol import Protocol

from .label import Label


class SubjectLabel(Label):
    template_name = None
    registered_subject_model = "edc_registration.registeredsubject"

    def __init__(self, subject_identifier: Optional[str] = None, site=None, **kwargs):
        super().__init__(**kwargs)
        self._registered_subject = None
        self.subject_identifier = subject_identifier
        self.site = site
        self.site_context = {}
        if site:
            self.site_context = {
                "site": str(self.site.id),
                "site_name": str(self.site.name),
                "site_title": str(self.site.siteprofile.title),
            }

    @property
    def registered_subject(self):
        if not self._registered_subject:
            model_cls = django_apps.get_model(self.registered_subject_model)
            self._registered_subject = model_cls.objects.get(
                subject_identifier=self.subject_identifier
            )
        return self._registered_subject

    @property
    def label_context(self) -> dict:
        context = {
            "protocol": Protocol().protocol,
            "subject_identifier": self.registered_subject.subject_identifier,
            "gender": self.registered_subject.gender,
            "dob": self.registered_subject.dob,
            "initials": self.registered_subject.initials,
            "identity": self.registered_subject.identity,
        }
        context.update(self.site_context)
        return context
